@@ -4,7 +4,7 @@
  * radiusd.h	Structures, prototypes and global variables
  *		for the FreeRADIUS server.
  *
- * Version:	$Id: radiusd.h,v 1.85 2001/05/04 16:58:04 aland Exp $
+ * Version:	$Id: radiusd.h,v 1.90 2001/07/28 00:50:40 cparker Exp $
  *
  */
 #include "libradius.h"
@@ -60,9 +60,13 @@ typedef struct auth_req {
 	int                     simul_mpp; /* WEIRD: 1 is false, 2 is true */
 
 	int			finished;
+	int			options; /* miscellanous options */
 	void			**container;
 } REQUEST;
 
+#define RAD_REQUEST_OPTION_NONE         (0)
+#define RAD_REQUEST_OPTION_LOGGED_CHILD (1 << 0)
+
 /*
  *  Function handler for requests.
  */
@@ -156,6 +160,7 @@ typedef int (*RAD_COMPARE_FUNC)(void *instance, VALUE_PAIR *, VALUE_PAIR *, VALU
  */
 extern const char	*progname;
 extern int		debug_flag;
+extern int		syslog_facility;
 extern const char	*radacct_dir;
 extern const char	*radlog_dir;
 extern const char	*radlib_dir;
@@ -197,6 +202,7 @@ int		session_zap(uint32_t nasaddr, int port, const char *user,
 void		debug_pair(FILE *, VALUE_PAIR *);
 int		log_err (char *);
 void		sig_cleanup(int);
+void		queue_sig_cleanup(int);
 int		rad_process(REQUEST *, int);
 int		rad_respond(REQUEST *, RAD_REQUEST_FUNP fun);
 
@@ -233,6 +239,7 @@ NAS		*nas_findbyname(char *nasname);
 void		version(void);
 
 /* log.c */
+int radlogdir_iswritable(char *);
 int		radlog(int, const char *, ...)
 #ifdef __GNUC__
 		__attribute__ ((format (printf, 2, 3)))
@@ -252,6 +259,7 @@ int proxy_send(REQUEST *request);
 char	*auth_name(char *buf, size_t buflen, REQUEST *request, int do_cli);
 int		rad_authenticate (REQUEST *);
 VALUE_PAIR	*rad_getpass(REQUEST *request);
+int             rad_check_return(VALUE_PAIR *list);
 int		rad_check_password(REQUEST *request);
 
 /* exec.c */
