@@ -1,7 +1,7 @@
 /*
  * auth.c	User authentication.
  *
- * Version:	$Id: auth.c,v 1.80 2001/04/12 15:26:46 aland Exp $
+ * Version:	$Id: auth.c,v 1.83 2001/07/28 00:42:15 cparker Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -22,7 +22,7 @@
  * Copyright 2000  Jeff Carneal <jeff@apex.net>
  */
 
-static const char rcsid[] = "$Id: auth.c,v 1.80 2001/04/12 15:26:46 aland Exp $";
+static const char rcsid[] = "$Id: auth.c,v 1.83 2001/07/28 00:42:15 cparker Exp $";
 
 #include "autoconf.h"
 #include "libradius.h"
@@ -179,6 +179,27 @@ static int rad_authlog(const char *msg, REQUEST *request, int goodpass) {
 	return 0;
 }
 
+/*
+ *       returncheck - Check for Auth-Type = Reject and return appropriate
+ *                     module return code if it is found.
+ */
+int rad_check_return(VALUE_PAIR *list)
+{
+	VALUE_PAIR	*authtype;
+
+	/* 
+	 * We check for Auth-Type = Reject here
+	 */
+
+	authtype = pairfind(list, PW_AUTHTYPE);
+	if((authtype) && authtype->lvalue == PW_AUTHTYPE_REJECT)  {
+		DEBUG2("rad_check_return:  Auth-Type is Reject");
+		return RLM_MODULE_REJECT;
+	}
+
+	return RLM_MODULE_UPDATED;
+}
+
 /*
  *	Check password.
  *
@@ -243,7 +264,7 @@ int rad_check_password(REQUEST *request)
 	 */
 	auth_item = request->password;
 	if (auth_item == NULL) {
-		DEBUG2("  auth: No password in the request");
+		DEBUG2("auth: No password in the request");
 		return -1;
 	}
 
@@ -273,7 +294,7 @@ int rad_check_password(REQUEST *request)
 
 	switch(auth_type) {
 		case PW_AUTHTYPE_CRYPT:
-			DEBUG2("  auth: Crypt");
+			DEBUG2("auth: type Crypt");
 			if (password_pair == NULL) {
 				result = auth_item->strvalue ? -1 : 0;
 				break;
@@ -284,7 +305,7 @@ int rad_check_password(REQUEST *request)
 				result = -1;
 			break;
 		case PW_AUTHTYPE_LOCAL:
-			DEBUG2("  auth: Local");
+			DEBUG2("auth: type Local");
 			/*
 			 *	Local password is just plain text.
 	 		 */
@@ -320,7 +341,7 @@ int rad_check_password(REQUEST *request)
 				result = -1;
 			break;
 		default:
-			DEBUG2("  auth: %s",
+			DEBUG2("auth: type \"%s\"",
 					dict_valbyattr(PW_AUTHTYPE, auth_type)->name);
 			/*
 			 *	See if there is a module that handles
@@ -458,7 +479,7 @@ int rad_authenticate(REQUEST *request)
 	 */
 	if (auth_item == NULL) {
 		if ((auth_item = pairfind(request->packet->vps, 
-					  PW_CHAP_PASSWORD)) != NULL) {
+				PW_CHAP_PASSWORD)) != NULL) {
 			password = "<CHAP-PASSWORD>";
 		
 		} else {
@@ -475,9 +496,9 @@ int rad_authenticate(REQUEST *request)
 	 */
 	r = module_authorize(request);
 	if (r != RLM_MODULE_NOTFOUND &&
-	    r != RLM_MODULE_NOOP &&
-	    r != RLM_MODULE_OK &&
-	    r != RLM_MODULE_UPDATED) {
+			r != RLM_MODULE_NOOP &&
+			r != RLM_MODULE_OK &&
+			r != RLM_MODULE_UPDATED) {
 		if (r != RLM_MODULE_FAIL && r != RLM_MODULE_HANDLED) {
 			rad_authlog("Invalid user", request, 0);
 			request->reply->code = PW_AUTHENTICATION_REJECT;
@@ -527,13 +548,13 @@ int rad_authenticate(REQUEST *request)
 	 */
 	if (result < 0) {
 
-		DEBUG2("  auth: Failed to validate the user.");
+		DEBUG2("auth: Failed to validate the user.");
 		request->reply->code = PW_AUTHENTICATION_REJECT;
 		
 		rad_authlog("Login incorrect", request, 0);
 
 		/* double check: maybe the secret is wrong? */
-		if ((debug_flag > 1) &&
+		if ((debug_flag > 1) && (auth_item != NULL) &&
 				(auth_item->attribute == PW_PASSWORD)) {
 			u_char *p;
 
@@ -577,8 +598,8 @@ int rad_authenticate(REQUEST *request)
 			request->reply->vps = tmp;
 
 			snprintf(logstr, sizeof(logstr), "Multiple logins (max %d) %s",
-				 check_item->lvalue,
-				 r == 2 ? "[MPP attempt]" : "");
+				check_item->lvalue,
+				r == 2 ? "[MPP attempt]" : "");
 			rad_authlog(logstr, request, 1);
 
 			result = -1;
@@ -650,7 +671,7 @@ int rad_authenticate(REQUEST *request)
 	 *	is NOT used anywhere below here, except for logging,
 	 *	so it should be safe...
 	 */
-	if (auth_item->attribute == PW_CHAP_PASSWORD) {
+	if ((auth_item != NULL) && (auth_item->attribute == PW_CHAP_PASSWORD)) {
 		password = "CHAP-Password";
 	}
 
