@@ -5,7 +5,7 @@
  *		write a decent parser. I know how to do that, really :)
  *		miquels@cistron.nl
  *
- * Version:	$Id: conffile.c,v 1.69 2001/04/11 02:44:11 cparker Exp $
+ * Version:	$Id: conffile.c,v 1.70 2001/07/01 19:18:19 cmiller Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -43,7 +43,7 @@
 #include "modules.h"
 
 static const char rcsid[] =
-"$Id: conffile.c,v 1.69 2001/04/11 02:44:11 cparker Exp $";
+"$Id: conffile.c,v 1.70 2001/07/01 19:18:19 cmiller Exp $";
 
 #define xstrdup strdup
 
@@ -172,7 +172,7 @@ static CONF_SECTION *cf_section_alloc(const char *name1, const char *name2,
 	memset(cs, 0, sizeof(CONF_SECTION));
 	cs->item.type = CONF_ITEM_SECTION;
 	cs->item.parent = parent;
-	cs->name1 = xstrdup(name1);
+	cs->name1 = strdup(name1);
 	cs->name2 = (name2 && *name2) ? xstrdup(name2) : NULL;
 
 	return cs;
