@@ -1,7 +1,7 @@
 /*
  * radiusd.c	Main loop of the radius server.
  *
- * Version:	$Id: radiusd.c,v 1.193 2001/05/04 16:58:04 aland Exp $
+ * Version:	$Id: radiusd.c,v 1.207 2001/07/24 16:04:13 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -27,7 +27,7 @@
 
 /* don't look here for the version, run radiusd -v or look in version.c */
 static const char rcsid[] =
-"$Id: radiusd.c,v 1.193 2001/05/04 16:58:04 aland Exp $";
+"$Id: radiusd.c,v 1.207 2001/07/24 16:04:13 aland Exp $";
 
 #include "autoconf.h"
 #include "libradius.h"
@@ -97,6 +97,7 @@ const char *radius_dir = NULL;
 const char *radacct_dir = NULL;
 const char *radlog_dir = NULL;
 const char *radlib_dir = NULL;
+int syslog_facility;
 int log_stripped_names;
 int debug_flag;
 int use_dbm = FALSE;
@@ -134,14 +135,15 @@ static gid_t server_gid;
 static const char *uid_name = NULL;
 static const char *gid_name = NULL;
 static int proxy_requests = TRUE;
+static int needs_child_cleanup = 0;
 int spawn_flag = TRUE;
-static struct rlimit core_limits;
 
 static void usage(void);
 
 static void sig_fatal (int);
 static void sig_hup (int);
 
+static int switch_users(void);
 static void rad_reject(REQUEST *request);
 static struct timeval *rad_clean_list(time_t curtime);
 static REQUEST *rad_check_list(REQUEST *);
@@ -191,6 +193,59 @@ static CONF_PARSER server_config[] = {
 	{ NULL, -1, 0, NULL, NULL }
 };
 
+
+static int switch_users() {
+	
+	/*
+	 *  Set the UID and GID, but only if we're NOT running
+	 *  in debugging mode.
+	 */
+	if (!debug_flag) {
+
+		/*  Set GID.  */
+		if (gid_name != NULL) {
+			struct group *gr;
+
+			gr = getgrnam(gid_name);
+			if (gr == NULL) {
+				if (errno == ENOMEM) {
+					radlog(L_ERR|L_CONS, "Cannot switch to Group %s: out of memory", gid_name);
+				} else {
+					radlog(L_ERR|L_CONS, "Cannot switch group; %s doesn't exist", gid_name);
+				}
+				exit(1);
+			}
+			server_gid = gr->gr_gid;
+			if (setgid(server_gid) < 0) {
+				radlog(L_ERR|L_CONS, "Failed setting Group to %s: %s", gid_name, strerror(errno));
+				exit(1);
+			}
+		}
+
+		/*  Set UID.  */
+		if (uid_name != NULL) {
+			struct passwd *pw;
+
+			pw = getpwnam(uid_name);
+			if (pw == NULL) {
+				if (errno == ENOMEM) {
+					radlog(L_ERR|L_CONS, "Cannot switch to User %s: out of memory", uid_name);
+				} else {
+					radlog(L_ERR|L_CONS, "Cannot switch user; %s doesn't exist", uid_name);
+				}
+				exit(1);
+			}
+			server_uid = pw->pw_uid;
+			if (setuid(server_uid) < 0) {
+				radlog(L_ERR|L_CONS, "Failed setting User to %s: %s", uid_name, strerror(errno));
+				exit(1);
+			}
+		}
+	}
+	return(0);
+}
+
+
 /*
  *  Read config files.
  */
@@ -198,6 +253,7 @@ static int reread_config(int reload)
 {
 	int pid = getpid();
 	CONF_SECTION *cs;
+	struct rlimit core_limits;
 
 	if (!reload) {
 		radlog(L_INFO, "Starting - reading configuration files ...");
@@ -212,9 +268,7 @@ static int reread_config(int reload)
 		return -1;
 	}
 
-	/*
-	 *  And parse the server's configuration values.
-	 */
+	/*  And parse the server's configuration values.  */
 	cs = cf_section_find(NULL);
 	if (cs == NULL) {
 		radlog(L_ERR|L_CONS, "No configuration information in radiusd.conf!");
@@ -222,9 +276,7 @@ static int reread_config(int reload)
 	}
 	cf_section_parse(cs, NULL, server_config);
 
-	/*
-	 *  Reload the modules.
-	 */
+	/*  Reload the modules.  */
 	DEBUG2("read_config_files:  entering modules setup");
 	if (setup_modules() < 0) {
 		radlog(L_ERR|L_CONS, "Errors setting up modules");
@@ -235,6 +287,13 @@ static int reread_config(int reload)
 	 *  Go update our behaviour, based on the configuration
 	 *  changes.
 	 */
+
+	/*  Get the current maximum for core files.  */
+	if (getrlimit(RLIMIT_CORE, &core_limits) < 0) {
+		radlog(L_ERR|L_CONS, "Failed to get current core limit:  %s", strerror(errno));
+		exit(1);
+	}
+
 	if (allow_core_dumps) {
 		if (setrlimit(RLIMIT_CORE, &core_limits) < 0) {
 			radlog(L_ERR|L_CONS, "Cannot update core dump limit: %s",
@@ -266,52 +325,14 @@ static int reread_config(int reload)
 		}
 	}
 
-	/*
-	 *  Set the UID and GID, but only if we're NOT running
-	 *  in debugging mode.
-	 */
-	if (!debug_flag) {
-
-		/*
-		 *  Set group.
-		 */
-		if (gid_name != NULL) {
-			struct group *gr;
-
-			gr = getgrnam(gid_name);
-			if (gr == NULL) {
-				radlog(L_ERR|L_CONS, "Cannot switch to Group %s: %s", gid_name, strerror(errno));
-				exit(1);
-			}
-			server_gid = gr->gr_gid;
-			if (setgid(server_gid) < 0) {
-				radlog(L_ERR|L_CONS, "Failed setting Group to %s: %s", gid_name, strerror(errno));
-				exit(1);
-			}
-		}
-
-		/*
-		 *  Set UID.
-		 */
-		if (uid_name != NULL) {
-			struct passwd *pw;
-
-			pw = getpwnam(uid_name);
-			if (pw == NULL) {
-				radlog(L_ERR|L_CONS, "Cannot switch to User %s: %s", uid_name, strerror(errno));
-				exit(1);
-			}
-			server_uid = pw->pw_uid;
-			if (setuid(server_uid) < 0) {
-				radlog(L_ERR|L_CONS, "Failed setting User to %s: %s", uid_name, strerror(errno));
-				exit(1);
-			}
-		}
+	if (reload) {
+		switch_users();  /* Don't do this yet, if we're just starting up. */
 	}
 
 	return 0;
 }
 
+
 /*
  *  Parse a string into a syslog facility level.
  */
@@ -427,7 +448,7 @@ static int str2fac(const char *s)
 	return LOG_DAEMON;
 }
 
-int main(int argc, char **argv)
+int main(int argc, char *argv[])
 {
 	REQUEST *request;
 	RADIUS_PACKET *packet;
@@ -444,11 +465,11 @@ int main(int argc, char **argv)
 	int fd = 0;
 	int devnull;
 	int status;
-	int syslog_facility = LOG_DAEMON;
 	int radius_port = 0;
 	struct servent *svp;
 	struct timeval *tv = NULL;
 
+	syslog_facility = LOG_DAEMON;
 #ifdef OSFC2
 	set_auth_parameters(argc,argv);
 #endif
@@ -473,9 +494,7 @@ int main(int argc, char **argv)
 	signal(SIGHUP, sig_hup);
 	signal(SIGINT, sig_fatal);
 	signal(SIGQUIT, sig_fatal);
-#if WITH_SNMP
 	signal(SIGPIPE, SIG_IGN);
-#endif
 #ifdef SIGTRAP
 	signal(SIGTRAP, sig_fatal);
 #endif
@@ -483,15 +502,10 @@ int main(int argc, char **argv)
 	signal(SIGIOT, sig_fatal);
 #endif
 
-	/*
-	 *  Pooled threads and child threads define their own
-	 *  signal handler.
+	/* this is right for threads, too, right?  
+	 * (Threads shouldn't get signals (from us) -- just be pthread_cancel()led.)
 	 */
-#ifndef WITH_THREAD_POOL
-#ifndef HAVE_PTHREAD_H
 	signal(SIGTERM, sig_fatal);
-#endif
-#endif
 	signal(SIGCHLD, sig_cleanup);
 #if 0
 	signal(SIGFPE, sig_fatal);
@@ -499,17 +513,13 @@ int main(int argc, char **argv)
 	signal(SIGILL, sig_fatal);
 #endif
 
-	/*
-	 *  Close unused file descriptors.
-	 */
+	/*  Close unused file descriptors.  */
 	for (t = 32; t >= 3; t--)
 		if(t!=devnull) 
 			close(t);
 
-	/*
-	 *  Process the options.
-	 */
-	while((argval = getopt(argc, argv, "Aa:bcd:fg:hi:l:p:sSvxXyz")) != EOF) {
+	/*  Process the options.  */
+	while ((argval = getopt(argc, argv, "Aa:bcd:fg:hi:l:p:sSvxXyz")) != EOF) {
 
 		switch(argval) {
 
@@ -614,27 +624,20 @@ int main(int argc, char **argv)
 		}
 	}
 
-	/*
-	 *  Get out PID: the configuration file reader uses it.
-	 */
+	/*  Get our PID: the configuration file reader uses it.  */
 	radius_pid = getpid();
 
-	/*
-	 *  Get the current maximum for core files.
-	 */
-	if (getrlimit(RLIMIT_CORE, &core_limits) < 0) {
-		radlog(L_ERR|L_CONS, "Failed to get current core limit:"
-				"  %s", strerror(errno));
-		exit(1);
-	}
-
-	/*
-	 *  Read the configuration files, BEFORE doing anything else.
-	 */
+	/*  Read the configuration files, BEFORE doing anything else.  */
 	if (reread_config(0) < 0) {
 		exit(1);
 	}
 
+	/*  We need root to do mkdir() and chown(), so we do this before giving up root.  -chad */
+	radlogdir_iswritable(uid_name); 
+
+	/*  Do this as soon as possible!  Make exceptions above here.  */
+	switch_users();
+
 #if HAVE_SYSLOG_H
 	/*
 	 *  If they asked for syslog, then give it to them.
@@ -647,9 +650,7 @@ int main(int argc, char **argv)
 	/* Do you want a warning if -g is used without a -l to activate it? */
 #endif
 
-	/*
-	 *  Initialize the request list.
-	 */
+	/*  Initialize the request list.  */
 	rl_init();
 
 	/*
@@ -701,7 +702,7 @@ int main(int argc, char **argv)
 	sa->sin_addr.s_addr = myip;
 	sa->sin_port = htons(auth_port);
 
-	result = bind (authfd, & salocal, sizeof(*sa));
+	result = bind (authfd, &salocal, sizeof(*sa));
 	if (result < 0) {
 		perror ("auth bind");
 		DEBUG("  There appears to be another RADIUS server already running on the authentication port UDP %d.", auth_port);
@@ -884,17 +885,12 @@ int main(int argc, char **argv)
 				buffer, auth_port, acct_port);
 	}
 
-	/*
-	 *  Note that we NO LONGER fork an accounting process!
-	 *  We used to do it for historical reasons, but that
-	 *  is no excuse...
-	 */
 	radlog(L_INFO, "Ready to process requests.");
 
 	/*
 	 *  Receive user requests
 	 */
-	for(;;) {
+	for (;;) {
 		if (need_reload) {
 			if (reread_config(TRUE) < 0) {
 				exit(1);
@@ -960,7 +956,7 @@ int main(int argc, char **argv)
 			 *  authfd and acctfd.  Check if we know
 			 *  this proxy for proxyfd.
 			 */
-			if(fd != proxyfd) {
+			if (fd != proxyfd) {
 				RADCLIENT *cl;
 				if ((cl = client_find(packet->src_ipaddr)) == NULL) {
 					radlog(L_ERR, "Ignoring request from unknown client %s:%d",
@@ -1013,6 +1009,7 @@ int main(int argc, char **argv)
 			request->number = request_num_counter++;
 			request->child_pid = NO_SUCH_CHILD_PID;
 			request->container = NULL;
+			request->options = RAD_REQUEST_OPTION_NONE;
 			strNcpy(request->secret, (char *)secret, sizeof(request->secret));
 			rad_process(request, spawn_flag);
 		} /* loop over authfd, acctfd, proxyfd */
@@ -1092,8 +1089,8 @@ int rad_process(REQUEST *request, int dospawn)
 			 *  and ignore them, if so.
 			 */
 			if (request->packet->sockfd != authfd) {
-				radlog(L_ERR, "Request packet code %d sent to authentication port from "
-					"client %s:%d - ID %d : IGNORED", request->packet->code,
+				radlog(L_ERR, "Authentication-Request sent to a non-authentication port from "
+					"client %s:%d - ID %d : IGNORED",
 					client_name(request->packet->src_ipaddr), request->packet->src_port,
 				request->packet->id);
 				request_free(&request);
@@ -1108,8 +1105,8 @@ int rad_process(REQUEST *request, int dospawn)
 			 *  and ignore them, if so.
 			 */
 			if (request->packet->sockfd != acctfd) {
-				radlog(L_ERR, "Request packet code %d sent to accounting port from "
-					"client %s:%d - ID %d : IGNORED", request->packet->code,
+				radlog(L_ERR, "Accounting-Request packet sent to a non-accounting port from "
+					"client %s:%d - ID %d : IGNORED",
 					client_name(request->packet->src_ipaddr), request->packet->src_port,
 					request->packet->id);
 				request_free(&request);
@@ -1127,7 +1124,7 @@ int rad_process(REQUEST *request, int dospawn)
 			 *  dropped.
 			 */
 			if (request->packet->sockfd != proxyfd) {
-				radlog(L_ERR, "Reply packet code %d sent to request port from "
+				radlog(L_ERR, "Reply packet code %d sent to a non-proxy reply port from "
 						"client %s:%d - ID %d : IGNORED", request->packet->code,
 						client_name(request->packet->src_ipaddr), request->packet->src_port,
 						request->packet->id);
@@ -1171,19 +1168,21 @@ int rad_process(REQUEST *request, int dospawn)
 	 *
 	 *  Build the reply template from the request template.
 	 */
-	if ((request->reply = rad_alloc(0)) == NULL) {
-		radlog(L_ERR, "No memory");
-		exit(1);
+	if (!request->reply) {
+		if ((request->reply = rad_alloc(0)) == NULL) {
+			radlog(L_ERR, "No memory");
+			exit(1);
+		}
+		request->reply->sockfd = request->packet->sockfd;
+		request->reply->dst_ipaddr = request->packet->src_ipaddr;
+		request->reply->dst_port = request->packet->src_port;
+		request->reply->id = request->packet->id;
+		request->reply->code = 0; /* UNKNOWN code */
+		memcpy(request->reply->vector, request->packet->vector, sizeof(request->reply->vector));
+		request->reply->vps = NULL;
+		request->reply->data = NULL;
+		request->reply->data_len = 0;
 	}
-	request->reply->sockfd = request->packet->sockfd;
-	request->reply->dst_ipaddr = request->packet->src_ipaddr;
-	request->reply->dst_port = request->packet->src_port;
-	request->reply->id = request->packet->id;
-	request->reply->code = 0; /* UNKNOWN code */
-	memcpy(request->reply->vector, request->packet->vector, sizeof(request->reply->vector));
-	request->reply->vps = NULL;
-	request->reply->data = NULL;
-	request->reply->data_len = 0;
 
 	/*
 	 *  If we're spawning a child thread, let it do all of
@@ -1362,8 +1361,15 @@ int rad_respond(REQUEST *request, RAD_REQUEST_FUNP fun)
 	if (request->proxy) {
 		int replicating;
 		replicating = proxy_receive(request);
-		if (replicating != 0) {
-			goto next_request;
+              switch (replicating) {
+                      case -1:
+                              /* on error just continue with next request */
+                              goto next_request;
+                      case 1:
+                              /* if this was a replicated request, mark it as
+                               * finished first, because it was postponed
+                               */
+                              goto finished_request;
 		}
 	}
 	
@@ -1419,8 +1425,8 @@ int rad_respond(REQUEST *request, RAD_REQUEST_FUNP fun)
 
 	/* Reprocess if we rejected last time */
 	if ((fun == rad_authenticate) &&
-			(request->reply->code == PW_AUTHENTICATION_REJECT) &&
-			(reprocess))  {
+	    (request->reply->code == PW_AUTHENTICATION_REJECT) &&
+	    (reprocess))  {
 		pairfree(&request->config_items);
 		(*fun)(request);
 	}
@@ -1440,11 +1446,12 @@ int rad_respond(REQUEST *request, RAD_REQUEST_FUNP fun)
 			 *  is NOT finished!
 			 */
 			if (proxy_sent == 1) {
+				finished = request->finished;
 				goto postpone_request;
 			}
 		}
 	} else if ((request->packet->code == PW_AUTHENTICATION_REQUEST) &&
-		(request->reply == NULL)) {
+		   (request->reply->code == 0)) {
 		/*
 		 *  We're not configured to reply to the packet,
 		 *  and we're not proxying, so the DEFAULT behaviour
@@ -1502,22 +1509,39 @@ finished_request:
 	 *  to make the server run more efficiently!
 	 */
 
-	/*  If we proxied this request, it's not safe to delete it until
-	 *  after the proxy reply
+	/*
+	 *  If we proxied this request, it's not safe to delete it until
+	 *  after the proxy reply.
+	 *
+	 *  However, we DO mark the request as no longer being handled
+	 *  by a thread.
 	 */
-	if (proxy_sent)
+	if (proxy_sent) {
+		finished = request->finished;
 		goto postpone_request;
+	}
 
-	if (request->packet && request->packet->vps) {
+	if (request->packet) {
 		pairfree(&request->packet->vps);
 		request->username = NULL;
 		request->password = NULL;
 	}
-	if (request->reply && request->reply->vps) {
+
+	/*
+	 *  If we've sent a reply to the NAS, then this request is
+	 *  pretty much finished, and we have no more need for any
+	 *  of the value-pair's in it, including the proxy stuff.
+	 */
+	if (request->reply->code != 0) {
 		pairfree(&request->reply->vps);
 	}
-
-	if (request->config_items) pairfree(&request->config_items);
+	pairfree(&request->config_items);
+	if (request->proxy) {
+		pairfree(&request->proxy->vps);
+	}
+	if (request->proxy_reply) {
+		pairfree(&request->proxy_reply->vps);
+	}
 
 	DEBUG2("Finished request %d", request->number);
 	finished = TRUE;
@@ -1529,12 +1553,19 @@ finished_request:
 next_request:
 	DEBUG2("Going to the next request");
 
-#if WITH_THREAD_POOL
+postpone_request:
+#if HAVE_PTHREAD_H
+	/*
+	 *  We are finished with the child thread.  The thread is detached,
+	 *  so that when it exits, there's nothing more for the server
+	 *  to do.
+	 *
+	 *  If we're running with thread pools, then this frees up the
+	 *  thread in the pool for another request.
+	 */
 	request->child_pid = NO_SUCH_CHILD_PID;
 #endif
 	request->finished = finished; /* do as the LAST thing before exiting */
-
-postpone_request:
 	return 0;
 }
 
@@ -1589,8 +1620,8 @@ static struct timeval *rad_clean_list(time_t now)
 	 *  too long...
 	 */
 	if ((last_tv_ptr != NULL) &&
-	(last_cleaned_list == now) &&
-	(tv.tv_sec != 0)) {		
+			(last_cleaned_list == now) &&
+			(tv.tv_sec != 0)) {		
 		int i;
 
 		/*
@@ -1753,12 +1784,20 @@ static REQUEST *rad_check_list(REQUEST *request)
 				 */
 			} else if (curreq->proxy != NULL) {
 				if (proxy_synchronous) {
-					DEBUG2("Sending duplicate proxy request to client %s:%d - ID: %d",
-							client_name(curreq->proxy->dst_ipaddr), request->packet->src_port,
-							curreq->proxy->id);
-
-					curreq->proxy_next_try = request->timestamp + proxy_retry_delay;
-					rad_send(curreq->proxy, curreq->proxysecret);
+					if (!curreq->proxy_reply) {
+						DEBUG2("Sending duplicate proxy request to client %s:%d - ID: %d",
+						       client_name(curreq->proxy->dst_ipaddr), request->packet->src_port,
+						       curreq->proxy->id);
+						
+						curreq->proxy_next_try = request->timestamp + proxy_retry_delay;
+						rad_send(curreq->proxy, curreq->proxysecret);
+					} else {
+						DEBUG2("Ignoring duplicate authentication packet"
+						       " from client %s:%d - ID: %d, as the proxy reply is currently being processed.",
+						       client_name(request->packet->src_ipaddr),
+						       request->packet->src_port,
+						       request->packet->id);
+					}
 				} else {
 					DEBUG2("Ignoring duplicate authentication packet"
 							" from client %s:%d - ID: %d, due to outstanding proxy request.",
@@ -1853,50 +1892,17 @@ typedef struct spawn_thread_t {
 	RAD_REQUEST_FUNP fun;
 } spawn_thread_t;
 
-/*
- *  If the child *thread* gets a termination signal,
- *  then exit from the thread.
- */
-static void sig_term(int sig)
-{
-	sig = sig;  /* -Wunused */
-	pthread_exit(NULL);
-}
-
 /*
  *  Spawn a new child thread to handle this request, and ONLY
  *  this request.
  */
 static void *rad_spawn_thread(void *arg)
 {
-	int replicating;
 	spawn_thread_t *data = (spawn_thread_t *)arg;
 	
-	/*
-	 *  Note that this behaviour only works on Linux.
-	 *
-	 *  It's generally NOT the thing to do, and should
-	 *  be fixed somehow.
-	 *
-	 *  Q: How do we signal a hung thread, and tell it to
-	 *  kill itself?
-	 */
-	signal(SIGTERM, sig_term);
-	
-	/*
-	 *  Keep only allowed attributes in the request.
-	 */
-	if (data->request->proxy) {
-		replicating = proxy_receive(data->request);
-		if (replicating != 0) {
-			data->request->finished = TRUE;
-			free(data);
-			return NULL;
-		}
-	}
-	
+	pthread_setcancelstate(PTHREAD_CANCEL_ASYNCHRONOUS, NULL);
+
 	rad_respond(data->request, data->fun);
-	data->request->child_pid = NO_SUCH_CHILD_PID;
 	free(data);
 	return NULL;
 }
@@ -1915,31 +1921,48 @@ static void *rad_spawn_thread(void *arg)
 static int rad_spawn_child(REQUEST *request, RAD_REQUEST_FUNP fun)
 {
 	child_pid_t		child_pid;
+	int retval = 0;
+
+	/* spawning and registering a child is a critical section, so
+	 * we refuse to handle SIGCHLDs normally until we're finished. */
+	signal(SIGCHLD, queue_sig_cleanup);
 
 #if HAVE_PTHREAD_H
-	int rcode;
-	spawn_thread_t *data;
+	{
+              int rcode;
+              spawn_thread_t *data;
+              pthread_attr_t attr;
 
-	data = (spawn_thread_t *) rad_malloc(sizeof(spawn_thread_t));
-	memset(data, 0, sizeof(data));
-	data->request = request;
-	data->fun = fun;
+              data = (spawn_thread_t *) rad_malloc(sizeof(spawn_thread_t));
+              memset(data, 0, sizeof(data));
+              data->request = request;
+              data->fun = fun;
 
-	/*
-	 *  Create a child thread, complaining on error.
-	 */
-	rcode = pthread_create(&child_pid, NULL, rad_spawn_thread, data);
-	if (rcode != 0) {
-		radlog(L_ERR, "Thread create failed for request from nas %s - ID: %d : %s",
-				nas_name2(request->packet), request->packet->id, strerror(errno));
-		return -1;
+              /*
+               *      Initialize the thread's attributes to detached.
+               *
+               *      We could call pthread_detach() later, but if
+               *      the thread exits between the create & detach
+               *      calls, it will need to be joined, which will
+               *      never happen.
+               */
+              pthread_attr_init(&attr);
+              pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_DETACHED);
+
+              /*
+               *  Create a child thread, complaining on error.
+               */
+              rcode = pthread_create(&request->child_pid, &attr,
+                                     rad_spawn_thread, data);
+              if (rcode != 0) {
+                      radlog(L_ERR, "Thread create failed for request from nas %s - ID: %d : %s",
+                             nas_name2(request->packet), request->packet->id,
+			     strerror(errno));
+                      retval = -1;
+                      goto exit_child_critsec;
+              }
+              pthread_attr_destroy(&attr);
 	}
-
-	/*
-	 *  Detach it, so it's state is automagically cleaned up on exit.
-	 */
-	pthread_detach(child_pid);
-
 #else
 	/*
 	 *  fork our child
@@ -1949,7 +1972,8 @@ static int rad_spawn_child(REQUEST *request, RAD_REQUEST_FUNP fun)
 		radlog(L_ERR, "Fork failed for request from nas %s - ID: %d",
 				nas_name2(request->packet),
 				request->packet->id);
-		return -1;
+		retval = -1;
+		goto exit_child_critsec;
 	}
 
 	if (child_pid == 0) {
@@ -1961,16 +1985,43 @@ static int rad_spawn_child(REQUEST *request, RAD_REQUEST_FUNP fun)
 		rad_respond(request, fun);
 		exit(0);
 	}
-#endif
 
 	/*
 	 *  Register the Child
 	 */
 	request->child_pid = child_pid;
-	return 0;
+#endif
+
+exit_child_critsec:
+	signal(SIGCHLD, sig_cleanup);
+	if (needs_child_cleanup > 0) {
+		sig_cleanup(0);
+	}
+	return retval;
 }
 #endif /* WITH_THREAD_POOL */
 
+
+#ifndef HAVE_PTHREAD_H
+static int sig_cleanup_walker(REQUEST *req, void *data)
+{
+	int pid = (int)data;
+		if ( req->child_pid != pid ) {
+			return RL_WALK_CONTINUE;
+		}
+	req->child_pid = NO_SUCH_CHILD_PID;
+	return 0;
+}
+#endif
+
+/* used in critical section */
+void queue_sig_cleanup(int sig) {
+	sig = sig; /* -Wunused */
+	needs_child_cleanup++;
+	return;
+}
+
+
 /*ARGSUSED*/
 void sig_cleanup(int sig)
 {
@@ -1984,6 +2035,8 @@ void sig_cleanup(int sig)
  
 	got_child = FALSE;
 
+	needs_child_cleanup = 0;  /* reset the queued cleanup number */
+
 	/*
 	 *  Reset the signal handler, if required.
 	 */
@@ -2005,7 +2058,8 @@ void sig_cleanup(int sig)
 		 *  process group, to prevent further attacks.
 		 */
 		if (debug_flag && (WIFSIGNALED(status))) {
-			radlog(L_ERR|L_CONS, "MASTER: Child PID %d failed to catch signal %d: killing all active servers.\n",
+			radlog(L_ERR|L_CONS, "MASTER: Child PID %d failed to catch "
+					"signal %d: killing all active servers.\n",
 					pid, WTERMSIG(status));
 			kill(0, SIGTERM);
 			exit(1);
@@ -2015,12 +2069,10 @@ void sig_cleanup(int sig)
 		 *  Loop over ALL of the active requests, looking
 		 *  for the one which caused the signal.
 		 */
-		for (curreq = rl_next(NULL); curreq != NULL; curreq = rl_next(curreq)) {
-			if (curreq->child_pid == pid) {
-				curreq->child_pid = NO_SUCH_CHILD_PID;
-				break;
-			}
+		if (rl_walk(sig_cleanup_walker, (void*)pid) != 0) {
+			radlog(L_ERR, "Failed to cleanup child %d", pid);
 		}
+
 	}
 #endif /* !defined HAVE_PTHREAD_H */
 }
@@ -2158,7 +2210,7 @@ static REQUEST *proxy_check_list(REQUEST *request)
 				request_free(&request);
 				return NULL;
 			}
-		} /* else there's no reply yet. */
+		}
 
 	} else {
 		/*
@@ -2247,9 +2299,15 @@ static int refresh_request(REQUEST *request, void *data)
 				 *  This request seems to have hung
 				 *   - kill it
 				 */
+#if WITH_THREAD_POOL || HAVE_PTHREAD_H
+				radlog(L_ERR, "Killing unresponsive thread for request %d",
+				       request->number);
+				pthread_cancel(child_pid);
+#else
 				radlog(L_ERR, "Killing unresponsive child %lu for request %d",
 				       child_pid, request->number);
-				child_kill(child_pid, SIGTERM);
+				kill(child_pid, SIGTERM);
+#endif
 			} /* else no proxy reply, quietly fail */
 		
 			/*
@@ -2262,9 +2320,17 @@ static int refresh_request(REQUEST *request, void *data)
 		 *	Maybe we haven't killed it.  In that case, print
 		 *	a warning.
 		 */
-		} else if (child_pid != NO_SUCH_CHILD_PID) {
-			radlog(L_ERR, "WARNING: Unresponsive child thread (pid %lu) for request %d",
+		} else if ((child_pid != NO_SUCH_CHILD_PID) &&
+			   ((request->options & RAD_REQUEST_OPTION_LOGGED_CHILD) == 0)) {
+			radlog(L_ERR, "WARNING: Unresponsive child (id %lu) for request %d",
 			       child_pid, number);
+
+			/*
+			 *  Set the option that we've sent a log message,
+			 *  so that we don't send more than one message
+			 *  per request.
+			 */
+			request->options |= RAD_REQUEST_OPTION_LOGGED_CHILD;
 		}
 		return RL_WALK_CONTINUE;
 	}
@@ -2280,9 +2346,14 @@ static int refresh_request(REQUEST *request, void *data)
 	if (!proxy_requests) goto setup_timeout;
 
 	/*
-	 *  We're proxying synchronously, so the retry_delay is zero.
+	 *  We're proxying synchronously, so we don't retry it here.
 	 *  Some other code takes care of retrying the proxy requests.
 	 */
+	if (proxy_synchronous) goto setup_timeout;
+
+	/*
+	 *  The proxy retry delay is zero, meaning don't retry.
+	 */
 	if (proxy_retry_delay == 0) goto setup_timeout;
 
 	/*
@@ -2328,8 +2399,7 @@ static int refresh_request(REQUEST *request, void *data)
 		delaypair = pairfind(request->proxy->vps, PW_ACCT_DELAY_TIME);
 		
 		if (!delaypair) {
-			delaypair = paircreate(PW_ACCT_DELAY_TIME,
-					PW_TYPE_INTEGER);
+			delaypair = paircreate(PW_ACCT_DELAY_TIME, PW_TYPE_INTEGER);
 			if (!delaypair) {
 				radlog(L_ERR|L_CONS, "no memory");
 				exit(1);
@@ -2342,7 +2412,15 @@ static int refresh_request(REQUEST *request, void *data)
 		free(request->proxy->data);
 		request->proxy->data = NULL;
 	} /* proxy accounting request */
-	
+
+	/*
+	 *  Assert that we have NOT seen the proxy reply yet.
+	 *
+	 *  If we HAVE seen it, then we SHOULD NOT be bugging the
+	 *  home server!
+	 */
+	assert(request->proxy_reply == NULL);
+
 	/*
 	 *  Send the proxy packet.
 	 */
