@@ -3,7 +3,7 @@
  *		Can also be installed as fingerd on the UNIX
  *		machine RADIUS runs on.
  *
- * Version:	$Id: radwho.c,v 1.26 2001/03/12 16:54:09 aland Exp $
+ * Version:	$Id: radwho.c,v 1.27 2001/06/18 22:53:52 cmiller Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -24,7 +24,7 @@
  */
 
 static const char rcsid[] =
-"$Id: radwho.c,v 1.26 2001/03/12 16:54:09 aland Exp $";
+"$Id: radwho.c,v 1.27 2001/06/18 22:53:52 cmiller Exp $";
 
 #include "autoconf.h"
 #include "libradius.h"
@@ -453,17 +453,19 @@ int main(int argc, char **argv)
 			if (ut.ut_user[0] && ut.ut_line[0]) {
 #endif
 #ifdef UT_HOSTSIZE
+
+#ifdef HAVE_UTMPX_H
+#	define UT_TIME ut_xtime
+#else
+#	define UT_TIME ut_time
+#endif
 			if (showname)
 				printf((rawoutput == 0? ufmt1: ufmt1r),
 						ut.ut_name,
 						fullname(ut.ut_name),
 						"shell",
 						ttyshort(ut.ut_line),
-#ifdef HAVE_UTMPX_H
-						dotime(ut.ut_xtime),
-#else
-						dotime(ut.ut_time),
-#endif
+						dotime(ut.UT_TIME),
 						ut.ut_host,
 						myname, eol);
 			else
@@ -471,11 +473,7 @@ int main(int argc, char **argv)
 						ut.ut_name,
 						ttyshort(ut.ut_line),
 						"shell",
-#ifdef HAVE_UTMPX_H
-						dotime(ut.ut_xtime),
-#else
-						dotime(ut.ut_time),
-#endif
+						dotime(ut.UT_TIME),
 						ut.ut_host,
 						myname, eol);
 #endif
