@@ -23,8 +23,14 @@
  * 16 Feb 2001, Hannu Laurila <hannu.laurila@japo.fi>
  * 	- LDAP<->RADIUS attribute mappings are now read from a file
  *  	- Support for generic RADIUS check and reply attribute.
+ * Jun 2001, Kostas Kalevras <kkalev@noc.ntua.gr>
+ *	- Fix: check and reply attributes from LDAP _replace_ existing ones
+ *	- Added "default_profile" directive, which points to radiusProfile 
+ *	  object, which contains default values for RADIUS users
+ *	- Added "profile_attribute" directive, which specifies user object 
+ *	  attribute pointing to radiusProfile object.
  */
-static const char rcsid[] = "$Id: rlm_ldap.c,v 1.36 2001/05/07 10:46:38 pam Exp $";
+static const char rcsid[] = "$Id: rlm_ldap.c,v 1.48 2001/07/17 09:17:00 pam Exp $";
 
 #include "autoconf.h"
 
@@ -66,12 +72,6 @@ struct TLDAP_RADIUS {
 typedef struct TLDAP_RADIUS TLDAP_RADIUS;
 
 #define MAX_SERVER_LINE 1024
-/*
- * These should really be in a module-specific data structure, which is
- * passed to the module with every request.
- */
-
-static struct timeval *timeout = NULL;
 
 typedef struct {
 	char           *server;
@@ -85,6 +85,8 @@ typedef struct {
 	char           *password;
 	char           *filter;
 	char           *basedn;
+	char           *default_profile;
+	char           *profile_attr;
 	char           *access_group;
 	char           *access_attr;
 	char           *dictionary_mapping;
@@ -99,16 +101,17 @@ static CONF_PARSER module_config[] = {
 	{"server", PW_TYPE_STRING_PTR, offsetof(ldap_instance,server), NULL, NULL},
 	{"port", PW_TYPE_INTEGER, offsetof(ldap_instance,port), NULL, "389"},
 	/* wait forever on network activity */
-	{"net_timeout", PW_TYPE_INTEGER, offsetof(ldap_instance,net_timeout.tv_sec), NULL, "-1"},
+	{"net_timeout", PW_TYPE_INTEGER, offsetof(ldap_instance,net_timeout.tv_sec), NULL, "10"},
 	/* wait forever for search results */
-	{"timeout", PW_TYPE_INTEGER, offsetof(ldap_instance,timeout.tv_sec), NULL, "-1"},
+	{"timeout", PW_TYPE_INTEGER, offsetof(ldap_instance,timeout.tv_sec), NULL, "20"},
 	/* allow server unlimited time for search (server-side limit) */
-	{"timelimit", PW_TYPE_INTEGER, offsetof(ldap_instance,timelimit), NULL, "-1"},
-
-	{"identity", PW_TYPE_STRING_PTR, offsetof(ldap_instance,login), NULL, NULL},
-	{"password", PW_TYPE_STRING_PTR, offsetof(ldap_instance,password), NULL, NULL},
+	{"timelimit", PW_TYPE_INTEGER, offsetof(ldap_instance,timelimit), NULL, "20"},
+	{"identity", PW_TYPE_STRING_PTR, offsetof(ldap_instance,login), NULL, ""},
+	{"password", PW_TYPE_STRING_PTR, offsetof(ldap_instance,password), NULL, ""},
 	{"basedn", PW_TYPE_STRING_PTR, offsetof(ldap_instance,basedn), NULL, NULL},
-	{"filter", PW_TYPE_STRING_PTR, offsetof(ldap_instance,filter), NULL, NULL},
+	{"filter", PW_TYPE_STRING_PTR, offsetof(ldap_instance,filter), NULL, "(uid=%u)"},
+	{"default_profile", PW_TYPE_STRING_PTR, offsetof(ldap_instance,default_profile), NULL, NULL},
+	{"profile_attribute", PW_TYPE_STRING_PTR, offsetof(ldap_instance,profile_attr), NULL, NULL},
 	{"access_group", PW_TYPE_STRING_PTR, offsetof(ldap_instance,access_group), NULL, NULL},
 	/* LDAP attribute name that controls remote access */
 	{"access_attr", PW_TYPE_STRING_PTR, offsetof(ldap_instance,access_attr), NULL, NULL},
@@ -126,7 +129,7 @@ static CONF_PARSER module_config[] = {
 #ifdef FIELDCPY
 static void     fieldcpy(char *, char **);
 #endif
-static VALUE_PAIR *ldap_pairget(LDAP *, LDAPMessage *, TLDAP_RADIUS *);
+static VALUE_PAIR *ldap_pairget(LDAP *, LDAPMessage *, TLDAP_RADIUS *,VALUE_PAIR **);
 static LDAP    *ldap_connect(void *instance, const char *, const char *, int, int *);
 static int     read_mappings(ldap_instance* inst);
 
@@ -149,6 +152,12 @@ ldap_instantiate(CONF_SECTION * conf, void **instance)
 		free(inst);
 		return -1;
 	}
+
+	if (inst->server == NULL) {
+		radlog(L_ERR, "rlm_ldap: missing 'server' directive.");
+		return -1;
+	}
+ 
 	inst->timeout.tv_usec = 0;
 	inst->net_timeout.tv_usec = 0;
 	inst->tls_mode = LDAP_OPT_X_TLS_TRY;
@@ -270,6 +279,7 @@ perform_search(void *instance, char *search_basedn, int scope, char *filter, cha
 	int		ldap_errno = 0;
 	ldap_instance  *inst = instance;
 
+	*result = NULL;
 	if (!inst->bound) {
 		DEBUG2("rlm_ldap: attempting LDAP reconnection");
 		if (inst->ld){
@@ -282,25 +292,23 @@ perform_search(void *instance, char *search_basedn, int scope, char *filter, cha
 		}
 		inst->bound = 1;
 	}
-	DEBUG2("rlm_ldap: performing search in %s, with filter %s", search_basedn, filter);
-	switch (ldap_search_st(inst->ld, search_basedn, scope, filter, attrs, 0, timeout, result)) {
+	DEBUG2("rlm_ldap: performing search in %s, with filter %s", search_basedn ? search_basedn : "(null)" , filter);
+	switch (ldap_search_st(inst->ld, search_basedn, scope, filter, attrs, 0, &(inst->timeout), result)) {
 	case LDAP_SUCCESS:
 		break;
 
 	default:
 		ldap_get_option(inst->ld, LDAP_OPT_ERROR_NUMBER, &ldap_errno);
 		radlog(L_ERR, "rlm_ldap: ldap_search() failed: %s", ldap_err2string(ldap_errno));
-/*
-		if(*result != NULL)
-			ldap_msgfree(*result);
-*/
 		inst->bound = 0;
+		ldap_msgfree(*result);	
 		return (RLM_MODULE_FAIL);
 	}
 
 	if ((ldap_count_entries(inst->ld, *result)) != 1) {
 		DEBUG("rlm_ldap: object not found or got ambiguous search result");
 		res = RLM_MODULE_NOTFOUND;
+		ldap_msgfree(*result);	
 	}
 	return res;
 }
@@ -315,17 +323,23 @@ perform_search(void *instance, char *search_basedn, int scope, char *filter, cha
 static int 
 ldap_authorize(void *instance, REQUEST * request)
 {
-	LDAPMessage    *result, *msg, *gr_result;
-	ldap_instance  *inst = instance;
-	char           *user_dn;
-	char           *attrs[] = {"*", NULL};
+	LDAPMessage	*result = NULL;
+	LDAPMessage	*msg = NULL;
+	LDAPMessage	*def_msg = NULL;
+	LDAPMessage	*def_attr_msg = NULL;
+	LDAPMessage	*gr_result = NULL;
+	LDAPMessage	*def_result = NULL;
+	LDAPMessage	*def_attr_result = NULL;
+	ldap_instance	*inst = instance;
+	char		*user_dn = NULL;
+	char		*attrs[] = {"*", NULL};
 	static char	filter[MAX_AUTH_QUERY_LEN];
-	static char 	*filt_patt = "(| (& (objectClass=GroupOfNames) (member=%{Ldap-UserDn})) (& (objectClass=GroupOfUniqueNames) (uniquemember=%{Ldap-UserDn})))"; 
-	VALUE_PAIR     *check_tmp;
-	VALUE_PAIR     *reply_tmp;
-	int             res;
-	VALUE_PAIR    **check_pairs, **reply_pairs;
-	char          **vals;
+	static char 	filt_patt[] = "(| (& (objectClass=GroupOfNames) (member=%{Ldap-UserDn})) (& (objectClass=GroupOfUniqueNames) (uniquemember=%{Ldap-UserDn})))"; 
+	VALUE_PAIR	*check_tmp;
+	VALUE_PAIR	*reply_tmp;
+	int		res;
+	VALUE_PAIR	**check_pairs, **reply_pairs;
+	char		**vals;
 
 	DEBUG("rlm_ldap: - authorize");
 
@@ -337,6 +351,7 @@ ldap_authorize(void *instance, REQUEST * request)
 	check_pairs = &request->config_items;
 	reply_pairs = &request->reply->vps;
 
+
 	/*
 	 * Check for valid input, zero length names not permitted
 	 */
@@ -370,6 +385,9 @@ ldap_authorize(void *instance, REQUEST * request)
 	 * given username
 	 */
 	pairadd(&request->packet->vps, pairmake("Ldap-UserDn", user_dn, T_OP_EQ));
+	ldap_memfree(user_dn);
+
+
 	/* Remote access is controled by attribute of the user object */
 	if (inst->access_attr) {
 		if ((vals = ldap_get_values(inst->ld, msg, inst->access_attr)) != NULL) {
@@ -377,8 +395,10 @@ ldap_authorize(void *instance, REQUEST * request)
 			if (!strncmp(vals[0], "FALSE", 5)) {
 				DEBUG("rlm_ldap: dialup access disabled");
 				ldap_msgfree(result);
+				ldap_value_free(vals);
 				return RLM_MODULE_USERLOCK;
 			}
+			ldap_value_free(vals);
 		} else {
 			DEBUG("rlm_ldap: no %s attribute - access denied by default", inst->access_attr);
 			ldap_msgfree(result);
@@ -396,20 +416,75 @@ ldap_authorize(void *instance, REQUEST * request)
 		if(!radius_xlat(filter, MAX_AUTH_QUERY_LEN, filt_patt,
 			        request, NULL)) 
 			radlog (L_ERR, "rlm_ldap: unable to create filter.\n"); 
- 
-		res = perform_search(instance, inst->access_group, LDAP_SCOPE_BASE, filter, NULL, &gr_result);
-		ldap_msgfree(gr_result);
 
-
-		if (res != RLM_MODULE_OK) {
+		if ((res = perform_search(instance, inst->access_group, LDAP_SCOPE_BASE, filter, NULL, &gr_result)) != RLM_MODULE_OK) {
+			ldap_msgfree(result);
 			if (res == RLM_MODULE_NOTFOUND)
 				return (RLM_MODULE_USERLOCK);
 			else
 				return (res);
+		} else 
+			ldap_msgfree(gr_result);
+	
+	}
+
+	/*
+	 * Check for the default profile entry. If it exists then add the 
+	 * attributes it contains in the check and reply pairs
+	 */
+
+	if (inst->default_profile){
+		if (radius_xlat(filter, MAX_AUTH_QUERY_LEN,
+				 "(objectclass=*)", request, NULL)){
+			if ((res = perform_search(instance, 
+				inst->default_profile, LDAP_SCOPE_BASE, 
+				filter, attrs, &def_result)) == RLM_MODULE_OK){
+				if ((def_msg = ldap_first_entry(inst->ld,def_result))){
+					if ((check_tmp = ldap_pairget(inst->ld,def_msg,inst->check_item_map,check_pairs)))
+						pairadd(check_pairs,check_tmp);
+					if ((reply_tmp = ldap_pairget(inst->ld,def_msg,inst->reply_item_map,reply_pairs)))
+						pairadd(reply_pairs,reply_tmp);
+				}
+				ldap_msgfree(def_result);
+			} else 
+				DEBUG("rlm_ldap: default_profile search failed");
+		} else 
+			DEBUG("rlm_ldap: unable to create filter for default profile.");
+	}
+
+	/*
+	 * Check for the profile attribute. If it exists, we assume that it 
+	 * contains the DN of an entry containg a profile for the user. That
+	 * way we can have different general profiles for various user groups
+	 * (students,faculty,staff etc)
+	 */
+
+	if (inst->profile_attr){
+		if ((vals = ldap_get_values(inst->ld, msg, inst->profile_attr)) != NULL) {
+			if (radius_xlat(filter, MAX_AUTH_QUERY_LEN,
+				 	"(objectclass=*)", request, NULL)){
+				if ((res = perform_search(instance, 
+					vals[0], LDAP_SCOPE_BASE, 
+					filter, attrs, &def_attr_result)) == RLM_MODULE_OK){
+					if ((def_attr_msg = ldap_first_entry(inst->ld,def_attr_result))){
+						if ((check_tmp = ldap_pairget(inst->ld,def_attr_msg,inst->check_item_map,check_pairs)))
+							pairadd(check_pairs,check_tmp);
+						if ((reply_tmp = ldap_pairget(inst->ld,def_attr_msg,inst->reply_item_map,reply_pairs)))
+							pairadd(reply_pairs,reply_tmp);
+					}
+					ldap_msgfree(def_attr_result);
+				} else 
+					DEBUG("rlm_ldap: profile_attribute search failed");
+			} else 
+				DEBUG("rlm_ldap: unable to create filter for general profile.");
+			ldap_value_free(vals);
 		}
 	}
+
+
 	DEBUG("rlm_ldap: looking for check items in directory...");
-	if ((check_tmp = ldap_pairget(inst->ld, msg, inst->check_item_map)) != NULL)
+
+	if ((check_tmp = ldap_pairget(inst->ld, msg, inst->check_item_map,check_pairs)) != NULL)
 		pairadd(check_pairs, check_tmp);
 
 
@@ -423,9 +498,11 @@ ldap_authorize(void *instance, REQUEST * request)
 
 	DEBUG("rlm_ldap: looking for reply items in directory...");
 
-	if ((reply_tmp = ldap_pairget(inst->ld, msg, inst->reply_item_map)) != NULL)
+
+	if ((reply_tmp = ldap_pairget(inst->ld, msg, inst->reply_item_map,reply_pairs)) != NULL)
 		pairadd(reply_pairs, reply_tmp);
 
+
 	DEBUG("rlm_ldap: user %s authorized to use remote access",
 	      request->username->strvalue);
 	ldap_msgfree(result);
@@ -471,7 +548,7 @@ ldap_authenticate(void *instance, REQUEST * request)
 		return RLM_MODULE_INVALID;
 	}
 
-	if (request->password->strvalue == 0) {
+	if (request->password->length == 0) {
 		radlog(L_ERR, "rlm_ldap: empty password supplied");
 		return RLM_MODULE_INVALID;
 	}
@@ -481,7 +558,7 @@ ldap_authenticate(void *instance, REQUEST * request)
 			request, NULL)) 
 		radlog (L_ERR, "rlm_ldap: unable to create filter.\n"); 
 
-	if ((vp_user_dn = pairfind(request->packet->vps, LDAP_USERDN)) == NULL) {
+	while((vp_user_dn = pairfind(request->packet->vps, LDAP_USERDN)) == NULL) {
 		if ((res = perform_search(instance, inst->basedn, LDAP_SCOPE_SUBTREE, filter, attrs, &result)) != RLM_MODULE_OK) {
 			return (res);
 		}
@@ -495,16 +572,16 @@ ldap_authenticate(void *instance, REQUEST * request)
 			return RLM_MODULE_FAIL;
 		}
 		pairadd(&request->packet->vps, pairmake("Ldap-UserDn", user_dn, T_OP_EQ));
+		ldap_memfree(user_dn);
 		ldap_msgfree(result);
-	} else {
-		user_dn = vp_user_dn->strvalue;
 	}
 
+	user_dn = vp_user_dn->strvalue;
+
 	DEBUG("rlm_ldap: user DN: %s", user_dn);
 
 	ld_user = ldap_connect(instance, user_dn, request->password->strvalue,
 			       1, &res);
-
 	if (ld_user == NULL)
 		return (res);
 
@@ -529,10 +606,10 @@ ldap_connect(void *instance, const char *dn, const char *password, int auth, int
 		*result = RLM_MODULE_FAIL;
 		return (NULL);
 	}
-	if (inst->net_timeout.tv_sec != -1 && ldap_set_option(ld, LDAP_OPT_NETWORK_TIMEOUT, (void *) &(inst->net_timeout)) != LDAP_OPT_SUCCESS) {
+	if (ldap_set_option(ld, LDAP_OPT_NETWORK_TIMEOUT, (void *) &(inst->net_timeout)) != LDAP_OPT_SUCCESS) {
 		radlog(L_ERR, "rlm_ldap: Could not set LDAP_OPT_NETWORK_TIMEOUT %ld.%ld", inst->net_timeout.tv_sec, inst->net_timeout.tv_usec);
 	}
-	if (inst->timelimit != -1 && ldap_set_option(ld, LDAP_OPT_TIMELIMIT, (void *) &(inst->timelimit)) != LDAP_OPT_SUCCESS) {
+	if (ldap_set_option(ld, LDAP_OPT_TIMELIMIT, (void *) &(inst->timelimit)) != LDAP_OPT_SUCCESS) {
 		radlog(L_ERR, "rlm_ldap: Could not set LDAP_OPT_TIMELIMIT %d", inst->timelimit);
 	}
 	if (inst->ldap_debug && ldap_set_option(NULL, LDAP_OPT_DEBUG_LEVEL, &(inst->ldap_debug)) != LDAP_OPT_SUCCESS) {
@@ -556,10 +633,7 @@ ldap_connect(void *instance, const char *dn, const char *password, int auth, int
 	}
 	DEBUG("rlm_ldap: waiting for bind result ...");
 
-	if (inst->timeout.tv_sec < 0)
-		rc = ldap_result(ld, msgid, 1, NULL, &res);
-	else
-		rc = ldap_result(ld, msgid, 1, &(inst->timeout), &res);
+	rc = ldap_result(ld, msgid, 1, &(inst->timeout), &res);
 
 	ldap_get_option(ld, LDAP_OPT_ERROR_NUMBER, &ldap_errno);
 	if(rc < 1) {
@@ -689,10 +763,8 @@ fieldcpy(char *string, char **uptr)
 
 static VALUE_PAIR *
 ldap_pairget(LDAP * ld, LDAPMessage * entry,
-	     TLDAP_RADIUS * item_map)
+	     TLDAP_RADIUS * item_map, VALUE_PAIR **pairs)
 {
-	BerElement     *berptr;
-	char           *attr;
 	char          **vals;
 	int             vals_count;
 	int             vals_idx;
@@ -701,76 +773,73 @@ ldap_pairget(LDAP * ld, LDAPMessage * entry,
 	int             token;
 	int             is_generic_attribute;
 	char            value[64];
-	VALUE_PAIR     *pairlist;
+	VALUE_PAIR     *pairlist = NULL;
 	VALUE_PAIR     *newpair = NULL;
-	pairlist = NULL;
-	if ((attr = ldap_first_attribute(ld, entry, &berptr)) == NULL) {
-		DEBUG("rlm_ldap: Object has no attributes");
-		return NULL;
-	}
-	do {
-		/* check if there is a mapping from this LDAP attribute to a RADIUS attribute */
-		for (element = item_map; element != NULL; element = element->next) {
-			if (!strcasecmp(attr, element->attr)) {
-                                /* mapping found, get the values */
-				if (((vals = ldap_get_values(ld, entry, attr)) == NULL)) {
-                                        DEBUG("rlm_ldap: ldap_get_values returned NULL for attribute %s", attr);
-                                        break;
-                                }
+	DICT_ATTR	*dattr;
 
+	/* check if there is a mapping from this LDAP attribute to a RADIUS attribute */
+	for (element = item_map; element != NULL; element = element->next) {
+	if ((vals = ldap_get_values(ld,entry,element->attr)) != NULL){
+			/* check whether this is a one-to-one-mapped ldap attribute or a generic
+			   attribute and set flag accordingly */
 
-				/* check whether this is a one-to-one-mapped ldap attribute or a generic
-				   attribute and set flag accordingly */
+			if (strcasecmp(element->radius_attr, GENERIC_ATTRIBUTE_ID)==0)
+				is_generic_attribute = 1;
+			else
+				is_generic_attribute = 0;
 
-				if (strcasecmp(element->radius_attr, GENERIC_ATTRIBUTE_ID)==0)
-					is_generic_attribute = 1;
-				else
-					is_generic_attribute = 0;
+			/* find out how many values there are for the attribute and extract all of them */
+
+			vals_count = ldap_count_values(vals);
+
+			for (vals_idx = 0; vals_idx < vals_count; vals_idx++) {
+				ptr = vals[vals_idx];
+
+				if (is_generic_attribute) {
+					/* this is a generic attribute */
+					int dummy; /* makes pairread happy */
 					
-                                /* find out how many values there are for the attribute and extract all of them */
-
-                                vals_count = ldap_count_values(vals);
-
-                                for (vals_idx = 0; vals_idx < vals_count; vals_idx++) {
-                                        ptr = vals[vals_idx];
-					
-					if (is_generic_attribute) {
-						/* this is a generic attribute */
-						int dummy; /* makes pairread happy */
-						
-						/* not sure if using pairread here is ok ... */
-						if ( (newpair = pairread(&ptr, &dummy)) != NULL) {
-							DEBUG("rlm_ldap: extracted attribute %s from generic item %s", 
-							      newpair->name, vals[vals_idx]);
-							pairadd(&pairlist, newpair);
-						} else {
-							radlog(L_ERR, "rlm_ldap: parsing %s failed: %s", 
-							       attr, vals[vals_idx]);
+					/* not sure if using pairread here is ok ... */
+					if ( (newpair = pairread(&ptr, &dummy)) != NULL) {
+						DEBUG("rlm_ldap: extracted attribute %s from generic item %s", 
+						      newpair->name, vals[vals_idx]);
+						if (! vals_idx){
+							dattr=dict_attrbyname(element->radius_attr);
+							if (dattr)
+								pairdelete(pairs,dattr->attr);
 						}
-					} else {
-						/* this is a one-to-one-mapped attribute */
-						token = gettoken(&ptr, value, sizeof(value));
-						if (token < T_EQSTART || token > T_EQEND) {
-							token = T_OP_EQ;
-						} else {
-							gettoken(&ptr, value, sizeof(value));
-						}
-						if (value[0] == 0) {
-							DEBUG("rlm_ldap: Attribute %s has no value", attr);
-							break;
-						}
-						DEBUG("rlm_ldap: Adding %s as %s, value %s & op=%d", attr, element->radius_attr, value, token);
-						if ((newpair = pairmake(element->radius_attr, value, token)) == NULL)
-							continue;
 						pairadd(&pairlist, newpair);
+					} else {
+						radlog(L_ERR, "rlm_ldap: parsing %s failed: %s", 
+						       element->attr, vals[vals_idx]);
 					}
+				} else {
+					/* this is a one-to-one-mapped attribute */
+					token = gettoken(&ptr, value, sizeof(value));
+					if (token < T_EQSTART || token > T_EQEND) {
+						token = T_OP_EQ;
+					} else {
+						gettoken(&ptr, value, sizeof(value));
+					}
+					if (value[0] == 0) {
+						DEBUG("rlm_ldap: Attribute %s has no value", element->attr);
+						break;
+					}
+					DEBUG("rlm_ldap: Adding %s as %s, value %s & op=%d", element->attr, element->radius_attr, value, token);
+						if ((newpair = pairmake(element->radius_attr, value, token)) == NULL)
+						continue;
+					if (! vals_idx){
+						dattr=dict_attrbyname(element->radius_attr);
+						if (dattr)
+							pairdelete(pairs,dattr->attr);
+					}
+					pairadd(&pairlist, newpair);
 				}
-                                ldap_value_free(vals);
-                        }
+			}
+			ldap_value_free(vals);
 		}
-	} while ((attr = ldap_next_attribute(ld, entry, berptr)) != NULL);
+	}
 
-	ber_free(berptr, 0);
 	return (pairlist);
 }
 
