@@ -2,7 +2,7 @@
  *  sql.c		rlm_sql - FreeRADIUS SQL Module
  *		Main code directly taken from ICRADIUS
  *
- * Version:	$Id: sql.c,v 1.39 2001/05/05 19:28:22 cmiller Exp $
+ * Version:	$Id: sql.c,v 1.41 2001/05/16 19:51:04 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -99,7 +99,7 @@ int sql_init_socketpool(SQL_INST * inst) {
 		sqlsocket->id = i;
 		sqlsocket->state = sockunconnected;
 
-#if HAVE_PTHREAD_H
+#if HAVE_SEMAPHORE_H
 		/*
 		 *  FIXME! Check return codes!
 		 */
@@ -150,7 +150,7 @@ int sql_close_socket(SQL_INST *inst, SQLSOCK * sqlsocket) {
 
 	radlog(L_DBG, "rlm_sql: Closing sqlsocket %d", sqlsocket->id);
 	(inst->module->sql_close)(sqlsocket, inst->config);
-#if HAVE_PTHREAD_H
+#if HAVE_SEMAPHORE_H
 	sem_destroy(sqlsocket->semaphore);
 #endif
 	free(sqlsocket);
@@ -167,7 +167,6 @@ int sql_close_socket(SQL_INST *inst, SQLSOCK * sqlsocket) {
  *************************************************************************/
 SQLSOCK * sql_get_socket(SQL_INST * inst) {
 	SQLSOCK *cur;
-	struct timeval timeout;
 	int tried_to_connect = 0;
 
 	while (inst->used == inst->config->num_sql_socks) {
@@ -193,13 +192,13 @@ SQLSOCK * sql_get_socket(SQL_INST * inst) {
 			continue;
 		}
 
-#if HAVE_PTHREAD_H
+#if HAVE_SEMAPHORE_H
 		if (sem_trywait(cur->semaphore) == 0) {
 #else
 		if (cur->in_use == SQLSOCK_UNLOCKED) {
 #endif
 			(inst->used)++;
-#ifndef HAVE_PTHREAD_H
+#ifndef HAVE_SEMAPHORE_H
 			cur->in_use = SQLSOCK_LOCKED;
 #endif
 			radlog(L_DBG, "rlm_sql: Reserving sql socket id: %d", cur->id);
@@ -222,7 +221,7 @@ SQLSOCK * sql_get_socket(SQL_INST * inst) {
 int sql_release_socket(SQL_INST * inst, SQLSOCK * sqlsocket) {
 
 	(inst->used)--;
-#if HAVE_PTHREAD_H
+#if HAVE_SEMAPHORE_H
 	sem_post(sqlsocket->semaphore);
 #else
 	sqlsocket->in_use = SQLSOCK_UNLOCKED;
@@ -303,7 +302,7 @@ int sql_getvpdata(SQL_INST * inst, SQLSOCK * sqlsocket, VALUE_PAIR **pair, char
 
 static int got_alrm;
 static void
-alrm_handler() {
+alrm_handler(void) {
 	got_alrm = 1;
 }
 
@@ -481,7 +480,7 @@ void query_log(SQL_INST * inst, char *querystr) {
 #if defined(F_LOCK) && !defined(BSD)
 			(void) lockf((int) sqlfile, (int) F_LOCK, (off_t) MAX_QUERY_LEN);
 #else
-			(void) flock(sqlfile, SQL_LOCK_EX);
+			(void) flock(sqlfile, LOCK_EX);
 #endif
 			fputs(querystr, sqlfile);
 			fputs(";\n", sqlfile);
